/** 
 * @(#)ActionServlet.java
 */
package framework.action;

import java.io.IOException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import framework.cache.Cache;
import framework.util.StringUtil;

/** 
 * Ʈѷ  ϴ   Ŭ̾Ʈ û ޾ ش ׼ Ѵ.
 * Ȯڰ (.do) Ǵ  û   óϱ Ͽ web.xml Ͽ  Ͽ ϸ
 *  ý Ѱ ü  ´.  
 * û  ׼Ű action.properties ActionŬ ã ü Ͽ Ͻ μ Ѵ. 
 */
public class ActionServlet extends HttpServlet {
	private static final long serialVersionUID = -6478697606075642071L;
	private static Log _logger = LogFactory.getLog(framework.action.ActionServlet.class);
	private static String _defaultServletName = null;
	private static final String[] _DEFAULT_SERVLET_NAMES = new String[] { "default", "WorkerServlet", "FileServlet", "resin-file", "SimpleFileServlet", "_ah_default" };

	/**
	 *  ü ʱȭ Ѵ.
	 * web.xml ʱȭ Ķͷ ϵǾ ִ action-mapping  ã ҽ  ϴ  Ѵ.
	 * 
	 * @param config ServletConfig ü
	 */
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		ResourceBundle bundle = null;
		try {
			bundle = ResourceBundle.getBundle(config.getInitParameter("action-mapping"));
			_defaultServletName = StringUtil.nullToBlankString(config.getInitParameter("default-servlet-name"));
			if ("".equals(_defaultServletName)) {
				for (String servletName : _DEFAULT_SERVLET_NAMES) {
					if (getServletContext().getNamedDispatcher(servletName) != null) {
						_defaultServletName = servletName;
						break;
					}
				}
			}
			if (getServletContext().getNamedDispatcher(_defaultServletName) == null) {
				throw new IllegalStateException("defaultServletName Error!");
			}
		} catch (MissingResourceException e) {
			throw new ServletException(e);
		}
		getServletContext().setAttribute("action-mapping", bundle);
		// Cache
		Cache.init();
	}

	/**
	 * Ŭ̾Ʈ Get  û  processRequest ó ̰Ѵ.
	 * 
	 * @param request HTTP Ŭ̾Ʈ ûü
	 * @param response HTTP Ŭ̾Ʈ 䰴ü
	 * 
	 * @exception java.io.IOException ActionServlet IO õ  ߻  
	 * @exception javax.servlet.ServletException  õ  ߻ 
	 */
	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		processRequest(request, response);
	}

	/**
	 * Ŭ̾Ʈ Post  û  processRequest ó ̰Ѵ.
	 * 
	 * @param request HTTP Ŭ̾Ʈ ûü
	 * @param response HTTP Ŭ̾Ʈ 䰴ü
	 * 
	 * @exception java.io.IOException ActionServlet IO õ  ߻  
	 * @exception javax.servlet.ServletException  õ  ߻ 
	 */
	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		processRequest(request, response);
	}

	/**
	 * Ŭ̾Ʈ Put  û  processRequest ó ̰Ѵ.
	 * 
	 * @param request HTTP Ŭ̾Ʈ ûü
	 * @param response HTTP Ŭ̾Ʈ 䰴ü
	 * 
	 * @exception java.io.IOException ActionServlet IO õ  ߻  
	 * @exception javax.servlet.ServletException  õ  ߻ 
	 */
	@Override
	protected void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	/**
	 * Ŭ̾Ʈ Delete  û  processRequest ó ̰Ѵ.
	 * 
	 * @param request HTTP Ŭ̾Ʈ ûü
	 * @param response HTTP Ŭ̾Ʈ 䰴ü
	 * 
	 * @exception java.io.IOException ActionServlet IO õ  ߻  
	 * @exception javax.servlet.ServletException  õ  ߻ 
	 */
	@Override
	protected void doDelete(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		processRequest(request, response);
	}

	//////////////////////////////////////////////////////////////////////////////////////////Private ޼ҵ
	private void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			String actionKey = getActionKey(request);
			String actionClassName = getActionClass(actionKey);
			Action action = null;
			if (actionClassName == null) {
				throw new PageNotFoundExeption("controller");
			} else {
				Class<?> actionClass = Class.forName(actionClassName);
				action = (Action) actionClass.newInstance();
				long currTime = 0;
				if (getLogger().isDebugEnabled()) {
					currTime = System.currentTimeMillis();
					getLogger().debug("ڡڡ " + request.getRemoteAddr() + "   \"" + request.getMethod() + " " + request.getRequestURI() + "\" û ۵Ǿϴ");
					getLogger().debug("ContentLength : " + request.getContentLength() + "bytes");
				}
				action.execute(this, request, response);
				if (getLogger().isDebugEnabled()) {
					getLogger().debug("١١ " + request.getRemoteAddr() + "   \"" + request.getMethod() + " " + request.getRequestURI() + "\" û Ǿϴ | duration : " + (System.currentTimeMillis() - currTime) + "ms\n");
				}
			}
		} catch (PageNotFoundExeption e) {
			this.getServletContext().getNamedDispatcher(_defaultServletName).forward(request, response);
		} catch (Exception e) {
			getLogger().error(e);
			throw new RuntimeException(e);
		}
	}

	private String getActionClass(String actionKey) {
		ResourceBundle bundle = (ResourceBundle) getServletContext().getAttribute("action-mapping");
		try {
			return ((String) bundle.getObject(actionKey)).trim();
		} catch (MissingResourceException e) {
			getLogger().error("error actionkey is " + actionKey);
			getLogger().error(e.getMessage());
			return null;
		}
	}

	private String getActionKey(HttpServletRequest request) {
		String path = request.getServletPath();
		int slash = path.lastIndexOf("/");
		int period = path.lastIndexOf(".");
		if (period > 0 && period > slash) {
			path = path.substring(0, period);
			return path;
		}
		return null;
	}

	private Log getLogger() {
		return ActionServlet._logger;
	}
}