/** 
 * @(#)OAuth10aUtil.java
 */
package framework.util;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;
import oauth.signpost.commonshttp.CommonsHttpOAuthProvider;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

/**
 * OAuth 1.0a  ϱ  ƿƼ Ŭ̴.
 */
public class OAuth10aUtil {

	/**
	 * , ܺο ü νϽȭ    
	 */
	private OAuth10aUtil() {
	}

	/**
	 * Result ü
	 */
	public static class Result {
		private int _statusCode;
		private String _content;

		public Result() {
			super();
		}

		public Result(int statusCode, String content) {
			super();
			this._statusCode = statusCode;
			this._content = content;
		}

		public int getStatusCode() {
			return _statusCode;
		}

		public String getContent() {
			return _content;
		}

		public String toString() {
			return String.format("Result={ statusCode : %d, content : %s }", getStatusCode(), getContent());
		}
	}

	/**
	 * Consumer ü
	 */
	public static class Consumer extends CommonsHttpOAuthConsumer {
		private static final long serialVersionUID = 3312085951191371927L;

		public Consumer(String consumerKey, String consumerSecret) {
			super(consumerKey, consumerSecret);
		}

		public Consumer(String consumerKey, String consumerSecret, String token, String tokenSecret) {
			this(consumerKey, consumerSecret);
			this.setTokenWithSecret(token, tokenSecret);
		}
	}

	/**
	 * Provider ü
	 */
	public static class Provider extends CommonsHttpOAuthProvider {
		private static final long serialVersionUID = -4670920617701598709L;

		public Provider(String requestTokenEndpointUrl, String accessTokenEndpointUrl, String authorizationWebsiteUrl) {
			super(requestTokenEndpointUrl, accessTokenEndpointUrl, authorizationWebsiteUrl);
			this.setOAuth10a(true);
		}
	}

	/**
	 * RequestToken û ܰ迡 ʿ Consumer Ѵ.
	 * @param consumerKey Ű
	 * @param consumerSecret ӽũ
	 * @return Consumer ü
	 */
	public static Consumer makeConsumer(String consumerKey, String consumerSecret) {
		return new Consumer(consumerKey, consumerSecret);
	}

	/**
	 * Protected Resource û ܰ迡 ʿ Consumer Ѵ.
	 * @param consumerKey Ű
	 * @param consumerSecret ӽũ
	 * @param token ׼ū
	 * @param tokenSecret ׼ūũ
	 * @return Consumer ü
	 */
	public static Consumer makeConsumer(String consumerKey, String consumerSecret, String token, String tokenSecret) {
		return new Consumer(consumerKey, consumerSecret, token, tokenSecret);
	}

	/**
	 * Է  Provider Ѵ.
	 * @param requestTokenEndpointUrl
	 * @param accessTokenEndpointUrl
	 * @param authorizationWebsiteUrl
	 * @return Provider ü
	 */
	public static Provider makeProvider(String requestTokenEndpointUrl, String accessTokenEndpointUrl, String authorizationWebsiteUrl) {
		return new Provider(requestTokenEndpointUrl, accessTokenEndpointUrl, authorizationWebsiteUrl);
	}

	/**
	 * Provider RequestToken ûϿ, RequestToken RequestTokenSecret ޾ƿ´. 
	 * @param consumer  ü
	 * @param provider ι̴ ü
	 * @param callbackUrl ݹּ
	 * @return authorize URL
	 */
	public static String getRequestToken(Consumer consumer, Provider provider, String callbackUrl) {
		try {
			String authorizeUrl = provider.retrieveRequestToken(consumer, callbackUrl);
			return authorizeUrl;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Provider AccessToken ûϿ, AccessToken AccessTokenSecret ޾ƿ´.
	 * @param consumer  ü
	 * @param provider ι̴ ü
	 * @param verifier  Ǵ ڵ
	 */
	public static void getAccessToken(Consumer consumer, Provider provider, String verifier) {
		try {
			provider.retrieveAccessToken(consumer, verifier);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Protected Resource  GET  ûѴ.
	 * @param consumer  ü
	 * @param url API URL
	 * @return Result ü
	 */
	public static Result get(Consumer consumer, String url) {
		return get(consumer, url, null);
	}

	/**
	 * Protected Resource  GET  ûѴ.
	 * @param consumer  ü
	 * @param url API URL
	 * @param headerMap 
	 * @return Result ü
	 */
	public static Result get(Consumer consumer, String url, Map<String, String> headerMap) {
		int statusCode = 0;
		String content = "";
		try {
			HttpClient client = new DefaultHttpClient();
			HttpGet get = new HttpGet(url);
			if (headerMap != null) {
				for (Entry<String, String> entry : headerMap.entrySet()) {
					get.addHeader(entry.getKey(), entry.getValue());
				}
			}
			consumer.sign(get);
			HttpResponse responseGet = client.execute(get);
			statusCode = responseGet.getStatusLine().getStatusCode();
			HttpEntity resEntityGet = responseGet.getEntity();
			if (resEntityGet != null) {
				content = EntityUtils.toString(resEntityGet);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new Result(statusCode, content);
	}

	/**
	 * Protected Resource  POST  ûѴ.
	 * @param consumer  ü
	 * @param url API URL
	 * @return Result ü
	 */
	public static Result post(Consumer consumer, String url) {
		return post(consumer, url, null, (Map<String, String>) null);
	}

	/**
	 * Protected Resource  POST  ûѴ.
	 * @param consumer  ü
	 * @param url API URL
	 * @param paramMap Ķ
	 * @return Result ü
	 */
	public static Result post(Consumer consumer, String url, Map<String, String> paramMap) {
		return post(consumer, url, paramMap, (Map<String, String>) null);
	}

	/**
	 * Protected Resource  POST  ûѴ.
	 * @param consumer  ü
	 * @param url API URL
	 * @param paramMap Ķ
	 * @param headerMap 
	 * @return Result ü
	 */
	public static Result post(Consumer consumer, String url, Map<String, String> paramMap, Map<String, String> headerMap) {
		int statusCode = 0;
		String content = "";
		try {
			HttpClient client = new DefaultHttpClient();
			HttpPost post = new HttpPost(url);
			if (headerMap != null) {
				for (Entry<String, String> entry : headerMap.entrySet()) {
					post.addHeader(entry.getKey(), entry.getValue());
				}
			}
			List<NameValuePair> params = new ArrayList<NameValuePair>();
			if (paramMap != null) {
				for (Entry<String, String> entry : paramMap.entrySet()) {
					params.add(new BasicNameValuePair(entry.getKey(), entry.getValue()));
				}
			}
			UrlEncodedFormEntity ent = new UrlEncodedFormEntity(params, "UTF-8");
			post.setEntity(ent);
			consumer.sign(post);
			HttpResponse responsePOST = client.execute(post);
			statusCode = responsePOST.getStatusLine().getStatusCode();
			HttpEntity resEntity = responsePOST.getEntity();
			if (resEntity != null) {
				content = EntityUtils.toString(resEntity);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new Result(statusCode, content);
	}

	/**
	 * Protected Resource  POST  ûѴ.
	 * @param consumer  ü
	 * @param url API URL
	 * @param paramMap Ķ
	 * @param fileList 
	 * @return Result ü
	 */
	public static Result post(Consumer consumer, String url, Map<String, String> paramMap, List<File> fileList) {
		return post(consumer, url, paramMap, fileList, null);
	}

	/**
	 * Protected Resource  POST  ûѴ.
	 * @param consumer  ü
	 * @param url API URL
	 * @param paramMap Ķ
	 * @param fileList 
	 * @param headerMap 
	 * @return Result ü
	 */
	public static Result post(Consumer consumer, String url, Map<String, String> paramMap, List<File> fileList, Map<String, String> headerMap) {
		int statusCode = 0;
		String content = "";
		try {
			HttpClient client = new DefaultHttpClient();
			HttpPost post = new HttpPost(url);
			if (headerMap != null) {
				for (Entry<String, String> entry : headerMap.entrySet()) {
					post.addHeader(entry.getKey(), entry.getValue());
				}
			}
			consumer.sign(post);
			MultipartEntity reqEntity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
			if (paramMap != null) {
				for (Entry<String, String> entry : paramMap.entrySet()) {
					reqEntity.addPart(entry.getKey(), new StringBody(entry.getValue()));
				}
			}
			if (fileList != null) {
				for (File file : fileList) {
					ContentBody contentBody = new FileBody(file);
					reqEntity.addPart("userfile", contentBody);
				}
			}
			post.setEntity(reqEntity);
			HttpResponse response = client.execute(post);
			statusCode = response.getStatusLine().getStatusCode();
			HttpEntity resEntity = response.getEntity();
			if (resEntity != null) {
				content = EntityUtils.toString(resEntity);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return new Result(statusCode, content);
	}
}
